/***************************************************************************
  $RCSfile: kchipcardcontrol.cpp,v $
                             -------------------
    cvs         : $Id: kchipcardcontrol.cpp,v 1.1 2003/05/12 04:19:19 aquamaniac Exp $
    begin       : Mon May 12 2003
    copyright   : (C) 2003 by Martin Preuss
    email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#include "kchipcardcontrol.h"
#include <chipcard.h>
#include <chameleon/directory.h>
#include <sys/types.h>
#include <signal.h>
#include <errno.h>
#include <string.h>

#include <qmessagebox.h>


KChipCardControl::KChipCardControl(QWidget* parent,
				   const char* name,
				   bool modal,
				   WFlags fl)
  :KChipCardControlBase(parent, name, modal, fl)
{
  QObject::connect((QObject*)but_stop,
		   SIGNAL(clicked()),
		   this,
		   SLOT(slotStop()));
  QObject::connect((QObject*)but_cont,
		   SIGNAL(clicked()),
		   this,
		   SLOT(slotContinue()));
  QObject::connect((QObject*)but_shutdown,
		   SIGNAL(clicked()),
		   this,
		   SLOT(slotShutdown()));
  QObject::connect((QObject*)but_reload,
		   SIGNAL(clicked()),
		   this,
		   SLOT(slotReload()));

}


KChipCardControl::~KChipCardControl(){
}


void KChipCardControl::slotStop(){
  if (_sendSignal(SIGTSTP))
    QMessageBox::information(this,
			     trUtf8("KChipCardControl"),
			     trUtf8("The chipcard daemon is suspended"),
			     QMessageBox::Ok | QMessageBox::Default,
			     QMessageBox::NoButton,
			     QMessageBox::NoButton);
}


void KChipCardControl::slotContinue(){
  if (_sendSignal(SIGCONT))
    QMessageBox::information(this,
			     trUtf8("KChipCardControl"),
			     trUtf8("The chipcard daemon has been restarted"),
			     QMessageBox::Ok | QMessageBox::Default,
			     QMessageBox::NoButton,
			     QMessageBox::NoButton);
}


void KChipCardControl::slotShutdown(){
  if (_sendSignal(SIGTERM))
    QMessageBox::information(this,
			     trUtf8("KChipCardControl"),
			     trUtf8("The chipcard daemon has been shut down"),
			     QMessageBox::Ok | QMessageBox::Default,
			     QMessageBox::NoButton,
			     QMessageBox::NoButton);
}


void KChipCardControl::slotReload(){
  _sendSignal(SIGHUP);
  if (_sendSignal(SIGHUP))
    QMessageBox::information(this,
			     trUtf8("KChipCardControl"),
			     trUtf8("The chipcard daemon has reloaded "
				    "its configuration files"),
			     QMessageBox::Ok | QMessageBox::Default,
			     QMessageBox::NoButton,
			     QMessageBox::NoButton);
}


bool KChipCardControl::_sendSignal(int signum){
  return _killallProcesses("chipcardd", signum);
}


bool KChipCardControl::_checkProcess(int pid, const char *n) {
  FILE *f;
  char nbuffer[256];
  char fbuffer[512];
  int i;
  string tmp;

  sprintf(nbuffer,"/proc/%d/cmdline",pid);
  f=fopen(nbuffer,"r");
  if (!f)
    return false;
  i=fread(fbuffer,1,sizeof(fbuffer)-1,f);
  if (i<1) {
    fclose(f);
    return false;
  }
  fclose(f);
  fbuffer[i]=0;
  tmp=string(fbuffer);
  if (tmp.find(n)!=string::npos)
    return true;
  return false;
}


bool KChipCardControl::_killallProcesses(const char *n, int sig) {
  DIRECTORYDATA *d;
  char nbuffer[256];
  int pid;
  bool killed;
  QString msg;

  killed=false;
  d=Directory_new();
  if (Directory_Open(d,"/proc")) {
    Directory_free(d);
    return 0;
  }
  while (!Directory_Read(d,
			 nbuffer,
			 sizeof(nbuffer))) {
    pid=atoi(nbuffer);
    if (pid!=0) {
      if (_checkProcess(pid, n)) {
	if (kill(pid,sig)) {
	  msg=trUtf8("Could not send signal to chipcardd:");
          msg+=strerror(errno);
	  QMessageBox::critical(this,
				trUtf8("KChipcardControl"),
				msg,
				QMessageBox::Ok | QMessageBox::Default,
				QMessageBox::NoButton,
				QMessageBox::NoButton);
	  return false;
	}
	else {
	  fprintf(stderr,"Signal send to process %d\n",pid);
	  killed=true;
	}
      }
    }
  } /* while */
  if (!killed) {
    QMessageBox::critical(this,
			  trUtf8("KChipcardControl"),
			  trUtf8("chipcardd seems not to be running"),
			  QMessageBox::Ok | QMessageBox::Default,
			  QMessageBox::NoButton,
			  QMessageBox::NoButton);
  }
  return killed;
}





