/***************************************************************************
 $RCSfile: dlgkvk.cpp,v $
                             -------------------
    cvs         : $Id: dlgkvk.cpp,v 1.5 2003/05/08 21:46:52 aquamaniac Exp $
    begin       : Wed Dec 26 2001
    copyright   : (C) 2001 by Martin Preuss
    email       : martin@libchipcard.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#include <qapplication.h>
#include <qmessagebox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qdatetime.h>
#include <qcombobox.h>
#include <qtextedit.h>
#include "dlgkvk.h"
#include "chipcard.h"


#ifndef trUtf8
# define trUtf8 tr
#endif



/*___________________________________________________________________________
 *AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
 *                           KMedicalCardTrader
 *YYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYY
 */


KMedicalCardTrader::CallBackResult KMedicalCardTrader::callback(bool first){

  qApp->processEvents();
  if (_abort)
    return CallBackAbort;
  return CallBackContinue;
}


KMedicalCardTrader::KMedicalCardTrader(QWidget *parent,
				       bool next,
				       unsigned int readerFlags,
				       unsigned int readerFlagsMask,
				       unsigned int status,
				       unsigned int statusMask,
				       unsigned int statusDelta)
:WaitForCardBase(parent,0,Qt::WType_Dialog)
,CTCardTrader(next, readerFlags, readerFlagsMask,
	      status, statusMask, statusDelta)
,_abort(false)
{
  QObject::connect((QObject*)but_abort,SIGNAL(clicked()),
		   this,SLOT(slotAbort()));

}


KMedicalCardTrader::~KMedicalCardTrader(){
}


void KMedicalCardTrader::slotAbort(){
  if (!_abort) {
    _abort=true;
    logText(trUtf8("User aborted"), true);
  }
}


bool KMedicalCardTrader::aborted(){
  return _abort;
}


CTError KMedicalCardTrader::start(){
  CTError err;

  _abort=false;
  err=CTCardTrader::start();
  if (!err.isOk())
    return err;
  show();
  raise();
  return err;
}


CTError KMedicalCardTrader::stop(){
  hide();
  return CTCardTrader::stop();
}


void KMedicalCardTrader::logText(const char *s, bool imp){
  QString st;
  QString nt;

  nt="<li>";
  if (imp)
    nt+="<font color=red>";
  nt+=s;
  if (imp)
    nt+="</font>";
  nt+="</li>";

  st="<qt><ul>"+_logText;
  st+=nt;
  st+="</ul></qt>";
  tv_log->setText(st);
  _logText+=nt;
  qApp->processEvents();
  fprintf(stderr,"LOG: %s\n", s);
}








/*___________________________________________________________________________
 *AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
 *                                 dlgKVK
 *YYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYY
 */



bool dlgKVK::init(){
  return true;
}


dlgKVK::dlgKVK(QWidget* parent,
               const char* name,
               bool modal,
               WFlags fl)
    :dlgKVKBase(parent,name,fl)
{
  QObject::connect((QObject*)but_read,SIGNAL(clicked()),this,SLOT(slotRead()));
  QObject::connect((QObject*)but_exit,SIGNAL(clicked()),this,SLOT(close()));
}


dlgKVK::~dlgKVK(){
}


bool dlgKVK::errorCheck(const string &title,
                        const string &errstr,
                        CTError err,
                        unsigned char ad) {
    if (!err.isOk(ad)) {
        QMessageBox mbox(title.c_str(),
                         (errstr+"\n("+err.errorString()+")").c_str(),
                         QMessageBox::Warning,
                         QMessageBox::Ok | QMessageBox::Default,
                         QMessageBox::NoButton,
                         QMessageBox::NoButton);
        mbox.exec();
        return false;
    }
    return true;
}


void dlgKVK::slotRead(){
  insuranceData data;
  string tmp;
  QDate d;
  CTError err;
  CTPointer<KMedicalCardTrader> trader;
  CTPointer<CTKVKCard> card;
  CTCard *cp;

  trader=new KMedicalCardTrader(this,
				false,
				0,
				0,
				CHIPCARD_STATUS_INSERTED,
				CHIPCARD_STATUS_INSERTED |
				CHIPCARD_STATUS_LOCKED_BY_OTHER,
				CHIPCARD_STATUS_INSERTED);
  err=trader.ref().start();
  if (!err.isOk()) {
    fprintf(stderr, "Could not initialize trader");
    if (err.code()==k_CTERROR_API &&
	(err.subcode1()==CHIPCARD_ERROR_NO_TRANSPORT ||
	 err.subcode1()==CHIPCARD_ERROR_NO_REQUEST)) {
      fprintf(stderr,
	      "Service unreachable, maybe \"chipcardd\" is not running?\n"
	     );
      QMessageBox::critical(this,
			    trUtf8("KMedicalCard"),
			    trUtf8("Service not reachable.\n"
				   "Maybe chipcardd is not running ?"),
			    QMessageBox::Ok | QMessageBox::Default,
			    QMessageBox::NoButton,
			    QMessageBox::NoButton);
    }
    else {
      QMessageBox::critical(this,
			    trUtf8("KMedicalCard"),
			    trUtf8("Could not start card trader.\n"
				   "Please check the console log"),
			    QMessageBox::Ok | QMessageBox::Default,
			    QMessageBox::NoButton,
			    QMessageBox::NoButton);
    }
    fprintf(stderr, "1:Could not mount card: %s\n",
	    err.errorString().c_str());
    return;
  }

  fprintf(stderr, "Please insert your card into any reader\n");
  // get the card

  while(1) {
    if (trader.ref().aborted()) {
      QMessageBox::critical(this,
			    trUtf8("KMedicalCard"),
			    trUtf8("User aborted."),
			    QMessageBox::Ok | QMessageBox::Default,
			    QMessageBox::NoButton,
			    QMessageBox::NoButton);
      trader.ref().stop();
      return;
    }
    trader.ref().logText(trUtf8("Checking for a card ..."), false);
    err=trader.ref().getNext(cp, 10);
    if (!err.isOk()) {
      if (err.code()==k_CTERROR_API &&
	  (err.subcode1()==CHIPCARD_ERROR_NO_TRANSPORT ||
	   err.subcode1()==CHIPCARD_ERROR_NO_REQUEST)) {
	fprintf(stderr,
		"Service unreachable, maybe \"chipcardd\" is not running?\n"
	       );
	trader.ref().logText(trUtf8("Service unreachable, "
				    "maybe \"chipcardd\" is not running?"),
			     true);
	QMessageBox::critical(this,
			      trUtf8("KMedicalCard"),
			      trUtf8("Service unreachable, "
				     "maybe \"chipcardd\" is not running?"),
			      QMessageBox::Ok | QMessageBox::Default,
			      QMessageBox::NoButton,
			      QMessageBox::NoButton);
	trader.ref().stop();
	return;
      }

    }
    else {
      // card found
      card=new CTKVKCard(*cp);
      delete cp;

      fprintf(stderr, "Not a German Medical Card");

      // check for memory card
      if (card.ref().isProcessorCard()) {
	trader.ref().logText(trUtf8("Card is not a German medical card."),
			     true);
	continue;
      }

      // try to open the card
      err=card.ref().openCard();
      if (!err.isOk()) {
	fprintf(stderr,"%s\n",err.errorString().c_str());
	trader.ref().logText(trUtf8("Card is not a German medical card."),
			     true);
	continue;
      }

      trader.ref().logText(trUtf8("German medical card found."), false);

      trader.ref().logText(trUtf8("Reading data."), false);
      err=card.ref().readCardData(data);
      if (!errorCheck(trUtf8("Chipcard").latin1(),
		      trUtf8("Could not read card data.").latin1(),
		      err)) {
	fprintf(stderr,
		"KMedicalCard: could not read card data:\n%s\n",
		err.errorString().c_str());
	trader.ref().stop();
	card.ref().closeCard();
	return;
      }
      if (!data.isValid) {
	fprintf(stderr,
		"KMedicalCard: card data is invalid\n%s\n",
		err.errorString().c_str());
	QMessageBox::critical(this,
			      trUtf8("KMedicalCard"),
			      trUtf8("Card is not a German medical card."),
			      QMessageBox::Ok | QMessageBox::Default,
			      QMessageBox::NoButton,
			      QMessageBox::NoButton);
	trader.ref().stop();
	card.ref().closeCard();
	return;
      }

      trader.ref().logText(trUtf8("Closing card."), false);
      err=card.ref().closeCard();
      if (!errorCheck(trUtf8("Chipcard").latin1(),
		      trUtf8("Could not close card.").latin1(),
		      err,0x62)){
	trader.ref().stop();
	return;
      }

      err=trader.ref().stop();
      if (!err.isOk()) {
	QMessageBox::critical(this,
			      trUtf8("KMedicalCard"),
			      trUtf8("Could not stop card trader\n."
				     "Please check the console log"),
			      QMessageBox::Ok | QMessageBox::Default,
			      QMessageBox::NoButton,
			      QMessageBox::NoButton);
	card.ref().closeCard();
	return;
      }

      // show data
      lab_companyname->setText(data.insuranceCompanyName.c_str());
      lab_companycode->setText(data.insuranceCompanyCode.c_str());
      lab_insnumber->setText(data.insuranceNumber.c_str());
      lab_insstate->setText(data.insuranceState.c_str());
      lab_title->setText(data.title.c_str());
      lab_forename->setText(data.forename.c_str());
      tmp=data.nameSuffix+data.name;
      lab_name->setText(tmp.c_str());
      if (data.dateOfBirth.length()<8)
	lab_birthdate->setText(data.dateOfBirth.c_str());
      else {
	d=QDate(atoi(data.dateOfBirth.substr(4,4).c_str()),
		atoi(data.dateOfBirth.substr(2,2).c_str()),
		atoi(data.dateOfBirth.substr(0,2).c_str()));
	lab_birthdate->setText(d.toString());
      }
      lab_street->setText(data.addrStreet.c_str());
      lab_state->setText(data.addrState.c_str());
      lab_zip->setText(data.addrPostalCode.c_str());
      lab_city->setText(data.addrCity.c_str());
      lab_bestbefore->setText(data.bestBefore.c_str());
      lab_number->setText(data.cardNumber.c_str());
      if (data.eastOrWest=="1")
	lab_west->setText(trUtf8("Western Germany"));
      else
	lab_west->setText(trUtf8("Eastern Germany"));
      lab_isvalid->setText(data.isValid?trUtf8("Yes"):trUtf8("No"));
      return;
    }
  } // while

}


